// $Id: basefile_plugin.h,v 1.2 2003/07/30 19:38:50 troein Exp $
//
// BioArray Software Environment (BASE) - homepage http://base.thep.lu.se/
// Copyright (C) 2002,2003 Lao Saal, Carl Troein, Johan Vallon-Christersson
//
// This file is part of BASE.
//
// BASE is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// BASE is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
#ifndef _BASEFILE_PLUGIN_H
#define _BASEFILE_PLUGIN_H

#include <basefile.h>

namespace BASE
{


class PluginBaseFileReader: BaseFileReader
{
  public:

	// Convenient shorthand
	typedef std::map<std::string, std::string> ssmap;
	typedef std::map<std::string, int> simap;
	typedef std::vector<std::string> strvec;
	typedef std::vector<int> intvec;

	// A struct describing an assay and its annotations. The annotations
	// are indexed from 0 to getAnnotationCount() - 1
	struct Assay
	{
		int id;
		std::string name;
		strvec annots;
	};

	PluginBaseFileReader(std::istream &s);
	~PluginBaseFileReader();

	// Tells the PluginBaseFileReader that plugin-specific settings can
	// be found in the named section. The existence of the settings named
	// in s will be verified by readInfoSections().
	bool registerSettings(const std::string &sectionname,
		const std::string &pluginversion, const strvec &s);
	// Get the value of a registered setting as a string, int or double.
	const std::string &getSetting(const std::string &name);
	int getSettingInt(const std::string &name);
	double getSettingDouble(const std::string &name);

	enum { format_any, format_serial, format_normal };
	typedef int format_t;

	// Tells the PluginBaseFileReader about the data format expected
	// by the plugin. readSpotsSection() will fail if the data format
	// doesn't fit the description here.
	bool registerDataFormat(const strvec &columns, const strvec &fields,
		bool allIntensities = true, int minCh = 0, int maxCh = 0,
		format_t format = format_any);

	// Reads plugins settings (if registerSettings() was called first)
	// and assay information.
	bool readInfoSections();
	// Reads a 'spots' section's headers and sets up everything needed
	// to loop over the data using readDataLine().
	bool readSpotsSection();

	// Returns the number of assays in current spots section
	int getAssayCount() { return _dataassays.size(); }
	// Returns info about an assay in the current spots section.
	const Assay &getAssay(int assay);
	// Returns the number of different assay annotations
	int getAnnotationCount() { return _annottypes.size(); }
	// Returns the name of an annotation
	const std::string &getAnnotationName(int annot);

	// Returns the number of channels in the current spots section
	int getChannelCount() { return _channels; }

	// Reads a single line of data from the spots section
	bool readDataLine();

	// Extracts the value for the named column from the
	// last read data line. name is a string reference to
	// avoid having to construct a new string for each call
	// (because a string is needed internally to find the
	// right column).
	const char *getColumn(const std::string &name);
	std::string getColumnString(const std::string &name);
	int getColumnInt(const std::string &name);
	double getColumnDouble(const std::string &name);

	// As above, but for a named field. The number of the assay
	// must be between 0 and getAssayCount() - 1.
	const char *getField(const std::string &name, int assay);
	std::string getFieldString(const std::string &name, int assay);
	int getFieldInt(const std::string &name, int assay);
	double getFieldDouble(const std::string &name, int assay);

	// Returns an intensity from the current data line. This function
	// is much more efficient than getDoubleField("intensityN", assay),
	// so use it when you can.
	double getIntensity(int assay, int channel);

	// The various errors getError() may return.
	enum
	{
		MISSINGSETTINGS = 100, BROKENSETTINGS, MISSINGASSAYS,
		BROKENASSAYS, MISSINGSPOTS, BROKENSPOTS, BADSPOTS,
		BADVERSION, BADCALLSEQUENCE, BADINDEXARGUMENT,
		LASTERR
	};

	// Overloaded from BaseFileReader
	const char *errText(int e);
	const char *errText() { return errText(_error); }
	int getError() { return _error; }

	// In some cases this will return a string with more information
	// about the error. Otherwise it'll just be empty.
	std::string errTextMore() { return _errdescr; };


  private:
	std::string toString(int i);

	// Plugin settings
	ssmap _settings;
	std::string _settingstype, _pluginversion;

	// Assay info and annotations
	strvec _annottypes;
	std::map<int, int> _assaynums;	// id -> number in vector
	std::vector<Assay> _assays;

	// Data format requirements
	strvec _reqcolumns, _reqfields;
	bool _reqallint, _reqserial, _reqnormal;
	int _channelsmin, _channelsmax;
	bool _datafmtregged;

	// Data matrix information
	int _channels;
	int _spotssection;
	int _assayDatacolumn;
	intvec _intensityfields;	// -1 for undefined
	simap _datacolumns, _datafields;
	intvec _dataassays;	// pos in header -> pos in _assays vector

	std::vector<char *> _databuf;

	// Error msgs
	static const char *_plerrtexts[];
	std::string _errdescr;

	// To avoid having to exit() in some cases
	static const std::string _undefstr;
	static const Assay _undefassay;

};


};	// end namespace BASE

#endif

